(({ behaviors, Toolshed: ts }) => {
  /**
   * Setup the Drupal behaviors for connecting Toolshed search filter forms.
   */
  behaviors.tsSearchFilterForm = {
    // Manage forms that are grouped together and combine filter.
    formGrps: new Map(),

    /**
     * Attach Toolshed filter forms
     *
     * @param {*} context
     * @param {*} settings
     */
    attach(context, settings) {
      ts.walkBySelector(context, '.toolshed-search-filter-form form', (form) => {
        // The target view data is likely in the parent DIV if not directly on
        // the form DOM element itself.
        const tv = form.dataset.targetView || form.parentNode.dataset.targetView;

        if (!(tv && 'GET' === form.method.toUpperCase())) return;

        form.dataset.targetView = tv;

        let list = this.formGrps.get(tv);
        if (!list) {
          this.formGrps.set(tv, list = []);
        }
        list.push(form);

        // Intercept the form submission to allow the combining of grouped filters.
        form.addEventListener('submit', this.onSubmit);
      });
    },

    /**
     *
     * @param {*} context
     * @param {object} settings
     * @param {string} trigger
     */
    detach(context, settings, trigger) {

    },

    /**
     * Form submission callback to combine search filters.
     *
     * @param {SubmitEvent} e
     *   Submit event for the form submission event.
     */
    onSubmit(e) {
      const form = e.target;
      const data = new FormData(form);
      const grp = behaviors.tsSearchFilterForm.formGrps.get(form.dataset.targetView||'');

      if (grp && grp.length) {
        grp.forEach((f) => {
          // Skip the current form values. Use the base form so these
          // existing values will have priority.
          if (f === form) return;

          const add = new FormData(f);
          for (const [k, v] of add.entries()) {
            if (k && v) data.append(k, v);
          }
        });

        // Submit the form with URL query string.
        const params = new URLSearchParams(data);
        window.location = ts.buildUrl(form.action, params.toString());
        e.preventDefault();
      }
    },
  };
})(Drupal);
